#-----------------------------------------------------------------------------
#  Copyright 2012-2013 3D-AlbertSoft ©
#  IPR (intellectual property rights) 3D-AlbertSoft
#  Permission to use this software for any purpose, and currently without fee, 
#  is hereby granted, Subject to compliance with all conditions of this license 
#  and copyright. 
#  Notice: This software is provided "as is" and without any express or implied.
#
#  License
#  This program can be used free of charge for commercial or non-commercial 
#  purposes (in each case without guarantees).
#  Is prohibited to alter, extract or publish parts of this code regardless of
#  their possible use.
#  Redistribution of this software is only permitted with the written consent 
#  of the author.
#  Make decryption processes for code of this software amounted to a breach of 
#  this license and intellectual property infringement, with legal implications
#  consecuententes under any circumstances authorizing decryption of this 
#  software.
#  
#  Version 0.8 - February 27 2013 ©
#
#  contact information: 3dalbertsoft@gmail.com
 
#-----------------------------------------------------------------------------
Sketchup::require 'sketchup'
Sketchup::require 'Helix-along-curve/translate-Helix'
#-----------------------------------------------------------------------------
module HelixD
  def self.entrada
     @version = "0.7.26"
     @len = Sketchup.get_locale.upcase
     model = Sketchup.active_model
     prompts = [@lw[1],@lw[2],@lw[3],@lw[4],@lw[5],@lw[6],@lw[7]+"         ",
               @lw[8],@lw[9],@lw[12],@lw[15],@lw[16]]
     
     defecto = model.get_attribute "Valor Defecto #{@len} #{@version}", "Lista Defecto"
     unless defecto
        defecto = [10.cm,10.cm,16.0,20.0,0.0,0.0,"0 (#{@lw[17]})","C (#{@lw[17]})",
                  @lw[11],@lw[14],2.cm,8]
     end
     list = [ "","","","","","","0 (#{@lw[17]})|1|2|3|4","A|B|C (#{@lw[17]})|D|E|F",
            @lw[11]+"|"+@lw[10],@lw[13]+"|"+@lw[14],"","" ]
     input = UI.inputbox prompts, defecto, list, @lw[0]
     return unless input
     input[0] = input[0].to_l
     input[1] = input[1].to_l
     input[2] = input[2].to_f.abs
     input[3] = input[3].to_f.abs
     if input[2]>0 && input[2]*input[3]<2
        input[3] = 2.0/ input[2]
     end
     input[4] = input[4].to_f
     input[5] = input[5].to_f.abs
     input[5] = 1.0 if input[5] > 1
     input[10] = input[10].abs.to_l
     if input[10] < 0.001.cm
        input[10] = 0.001.cm
        input[11] = 4
     end
     input[11] = input[11].to_i.abs
     input[11] = 3 if input[11] < 3
     return input
  end
  #-----------------------------------------------------------------------------
  def self.b_spline_apro(order, t, nbpts, knot)
     basis = []
     kmax = nbpts + order - 1
     for i in 0..(kmax-1)
        basis [i] = (t >= knot[i] && t < knot[i+1]) ? 1.0 : 0.0
     end  
     for k in 1..(order-1)
        for i in 0..(kmax-k-1)
           d = (basis[i] == 0.0) ? 0 : ((t-knot[i])*basis [i])/ (knot[i+k]-knot[i])
           e = (basis[i+1] == 0.0) ? 0 : ((knot[i+k+1]-t)*basis[i+1])/ (knot[i+k+1]-knot[i+1])
           basis[i] = d + e
        end
     end
     basis [nbpts-1] = 1.0 if t == knot[kmax]
     return basis
  end
  #---------------------------------------------------------------------------
  # Quitar puntos que petenecen a una misma recta
  def self.quitar_colineales(pts)
     ptb=pts
     if @closed
        ptb = ptb|ptb
        ptb[ptb.length] = ptb[0]
     else
        ptb = ptb|ptb
     end
     pts.clear
     vector1 = Geom::Vector3d.new
     vector2 = Geom::Vector3d.new
     vector3 = Geom::Vector3d.new 
     pts[0] = ptb[0]
     for q in 1..ptb.length-2
        vector1 = ptb[q-1].vector_to ptb[q]
        vector2 = ptb[q].vector_to ptb[q+1]
        vector3 = vector1*vector2
        pts << ptb[q] if vector3.length>0
     end
     pts[pts.length] = ptb[ptb.length-1]
     if @closed
        vector1 = pts[0].vector_to pts[1]
        vector2 = pts[0].vector_to pts[pts.size-2]
        vector3 = vector1*vector2
        unless vector3.length>0
           pts.delete_at(0)
           pts.delete_at(pts.size-1)
           pts[pts.size]=pts[0] 
        end
     end
     return pts
  end
  #-----------------------------------------------------------------------------
  # Obtener intervalos regulares
  def self.p_regulares1(pts)
     # longitud de curva
     long_c = 0
     for w in 0..pts.length-2
        distance = pts[w].distance pts[w+1]
        long_c = long_c + distance
     end
     # Obtener intervalos regulares
     tramo = long_c.to_f / @seg_b_spline
     if tramo<0.00028
        @seg_b_spline = (long_c/ 0.00028).to_i
        return if @seg_b_spline<1
        tramo = long_c.to_f / @seg_b_spline
     end
     ptd = []
     ptd.clear
     ptd[0] = pts[0]
     i1 = 0
     i2 = tramo
     residuo = 0
     for l in 0..pts.length-2
        separacion = pts[l+1].distance pts[l]
        i2 = i2 - separacion
        # redondeo de i2
        a1=((i2-i2.to_i)*1000000000000000).to_i.to_s.length
        if a1<9
           i2 = i2.to_i
       end
        d1 = separacion/ tramo
        p_paso = 1/ d1
        if i2<=0
           porc = -i2/ separacion
           veces = -i2/ tramo
           for m in 0..veces
              ap = porc-p_paso*m
              ptd[i1+1] = Geom::Point3d.linear_combination ap,pts[l],1-ap,pts[l+1]
              i1+=1
           end
           residuo = ap*separacion
           i2 = tramo-residuo
        end
     end
     ptd[@seg_b_spline] = Geom::Point3d.new pts[pts.size-1]
     return ptd
  end
  #-----------------------------------------------------------------------------
  def self.p_regulares2(pts)
     # longitud de curva
     long_c = 0
     for w in 0..pts.length-2
        distance = pts[w].distance pts[w+1]
        long_c = long_c + distance
     end
     # Obtener intervalos regulares
     tramo = long_c.to_f / @new_segment
     if tramo<0.00028
        @new_segment = (long_c/ 0.00028).to_i
        return if @new_segment<1
        tramo = long_c.to_f / @new_segment
     end
     ptd = []
     ptd.clear
     ptd[0] = pts[0]
     i1 = 0
     i2 = tramo
     residuo = 0
     for l in 0..pts.length-2
        separacion = pts[l+1].distance pts[l]
        i2 = i2 - separacion
        d1 = separacion/ tramo
        p_paso = 1/ d1
        if i2<=0
           porc = -i2/ separacion
           veces = -i2/ tramo
           for m in 0..veces
              ap = porc-p_paso*m
              ptd[i1+1] = Geom::Point3d.linear_combination ap,pts[l],1-ap,pts[l+1]
              i1+=1
           end
           residuo = ap*separacion
           i2 = tramo-residuo
        end
     end
     ptd[@new_segment] = Geom::Point3d.new pts[pts.length-1]
     return ptd
  end
  #-----------------------------------------------------------------------------
  def self.dibujar_helice
     model = Sketchup.active_model
     ent=model.active_entities
     sel=model.selection
     TraducirHelix
     @lw = TraducirHelix.hallar_traduccion
     selec_curva = false
     sel.each {|i|
        if i.is_a?(Sketchup::Edge)
           selec_curva=true
           break
        end
     }
     # verificar que la seleccion sea correcta
     unless selec_curva
        UI.messagebox @lw[18]
        return
     end
     
     input = self.entrada
     return unless input
     r1,r2,vueltas,seg_por_vuelta,fase,random_suave,adherencia,calidad,sentido_del_giro,crear_t,radio_tubo,lados_tubo = input
     
     fase = fase.degrees
     random_suave = random_suave*0.5
     apego = adherencia[0].chr.to_i
     if calidad == "A"
        num_seg = 20
        seg_1b_spline = 0.3
     elsif calidad == "B"
        num_seg = 20
        seg_1b_spline = 3
     elsif calidad == "C (#{@lw[17]})"
        num_seg = 12
        seg_1b_spline = 6
     elsif calidad == "D"
        num_seg = 15
        seg_1b_spline = 12
     elsif calidad == "E"
        num_seg = 20
        seg_1b_spline = 20
     elsif calidad == "F"
        num_seg = 16
        seg_1b_spline = 25
     end
     sentido_g = -1
     sentido_g = 1 if sentido_del_giro == @lw[11]
     crear_tubo = false
     crear_tubo = true if crear_t == @lw[13]
     
     Sketchup::set_status_text @lw[19]+'  | * - - - - |', SB_PROMPT
     order = 3
     vuel = vueltas
     
     # obtener y soldar curva
     entidades=[]
     sl=sel.length
     verts=[], edges=[], newVerts=[]
     edges.clear
     verts.clear
     newVerts.clear
     startEdge=startVert=nil
     sel.each {|item| edges.push(item) if item.is_a?(Sketchup::Edge) }
     edges.each {|edge| verts.push(edge.vertices)}
     verts.flatten!
     
     #FIND AN END VERTEX
     vertsShort=[], vertsLong=[]
     vertsShort.clear
     vertsLong.clear
     verts.each do |v|
        if vertsLong.include?(v)
           vertsShort.push(v)
        else
           vertsLong.push(v)
        end
     end # each
     if (startVert=(vertsLong-vertsShort).first)==nil
        startVert=vertsLong.first
        @closed=true
        startEdge = startVert.edges.first
     else
        @closed=false
        startEdge = (edges & startVert.edges).first
     end
     
     #SORT VERTICES, LIMITING TO THOSE IN THE SELECTION SET
     if startVert==startEdge.start
        newVerts=[startVert]
        counter=0
        while newVerts.length < verts.length
           edges.each do |edge|
              if edge.end==newVerts.last
                 newVerts.push(edge.start)
              elsif edge.start==newVerts.last
                 newVerts.push(edge.end)
              end
           end # each
           counter+=1
           if counter > verts.length
              newVerts.reverse!
              reversed=true
           end
        end
     else
        newVerts=[startVert]
        counter=0
        while newVerts.length < verts.length
           edges.each do |edge|
              if edge.end==newVerts.last
                 newVerts.push(edge.start)
              elsif edge.start==newVerts.last
                 newVerts.push(edge.end)
              end
           end
           counter+=1
           if counter > verts.length
              newVerts.reverse!
              reversed=true
           end
        end
     end
     
     if @closed
        ultimo_vert = newVerts[newVerts.size/ 2]
        newVerts.uniq!
        newVerts[newVerts.size]=ultimo_vert
     else
        newVerts.uniq!
     end
     
     newVerts.reverse! if reversed
     newVerts[0].position
     pts=[]
     pts.clear
     pts<<(newVerts.collect{|v| Geom::Point3d.new(v.position)})
     pts.flatten!
     
     pts = self.quitar_colineales(pts)
      
     es_linea = true
     es_linea = false if pts.size > 2
     
     # Dividir segmentos rectos en partes iguales
     if apego > 0
        pti = []
        pti.clear
        npi = 1.0/ (apego+1)  # numero de puntos intermedios
        for i in 0..pts.size-2
           pti << pts[i]
           for k in 1..apego.to_f
              pti << Geom::Point3d.linear_combination((1-npi*k),pts[i],npi*k,pts[i+1])
           end
        end
        pti << pts[pts.size-1]
        pts=pti
     end
     
     # Condicion de frontera para curva cerrada
     if @closed
        punto_ini = (Geom::Point3d.linear_combination 0.5,pts[0],0.5,pts[1])
        pts[0] = punto_ini
        pts << punto_ini
     end
     @new_segment = pts.size.to_i*seg_por_vuelta/ 3.0
     @new_segment = vueltas*seg_por_vuelta if vueltas != 0 
     @new_segment = pts.size.to_i*seg_por_vuelta/ 3.0 if r1==0 && r2==0
     #-----------------------------------------------------------------------------
     # Implementacion b-spline
     order = pts.length if order > pts.length
     #initialization
     curve = []
     curve.clear
     nbpts = pts.length
     kmax = nbpts + order - 1
     #Generating the uniform open knot vector
     knot = []
     knot[0] = 0.0
     for i in 1..kmax
        knot[i] = ((i >= order) && (i < nbpts + 1)) ? knot[i-1] + 1.0 : knot[i-1]
     end
     
     #calculate the points of the B-Spline curve
     t = 0.0
     @seg_b_spline = (seg_1b_spline*(pts.size-1)).round
     @seg_b_spline = 1 if @seg_b_spline < 1
     step = knot[kmax] / @seg_b_spline
     
     # given a nt integer (number of segments to interpolate) 
     # interpolate nt points of a segment
     for icrv in 0..@seg_b_spline   
        #calculate parameter t
        t = knot[kmax] if (knot[kmax] - t) < 0.0000001
        #calculate the basis
        basis = self.b_spline_apro order, t, nbpts, knot
        #Loop on the control points
        pt = Geom::Point3d.new
        pt.x = pt.y = pt.z = 0.0
        for i in 0..(nbpts-1)
           pt.x += basis[i] * pts[i].x
           pt.y += basis[i] * pts[i].y
           pt.z += basis[i] * pts[i].z
        end
        curve.push pt
        t += step
     end
     pts_s = curve
     pts.clear
     pts = self.p_regulares1(pts_s)
     
     # Implementacion de splines cubicas naturales 3d
     x=[], y=[], z=[], omega=[], delta=[], der=[]
     for i in 0..pts.size-1
        x[i] = pts[i][0]
        y[i] = pts[i][1]
        z[i] = pts[i][2]
     end
     m = pts.size-1; # m es el numero de intervalos que tendremos 
     
     # Calculamos el valor de omega (sera el mismo en X,Y y en Z)
     omega[0] = 0.5
     for i in 1..m-1
        omega[i] = 1.0/(4-omega[i-1])
     end
     omega[m] = 1.0/(2-omega[m-1])
     
     #----------------------------------------------------------x
     # Calculamos el valor de delta para X 
     delta[0] = 3*(x[1]-x[0])*omega[0]
     for i in 1..m-1
        delta[i] = (3*(x[i+1]-x[i-1])-delta[i-1])*omega[i]
     end
     delta[m] = (3*(x[m]-x[m-1])-delta[m-1])*omega[m]
     # Valor de la primera derivada en los puntos (eje X) 
     der[m] = delta[m]
     for i in 0..m-1
        der[m-1-i] = delta[m-1-i]-omega[m-1-i]*der[m-i]
     end
     ax=[], bx=[], cx=[], dx=[]
     # Sustituimos los valores de omega, delta y la primera derivada
     # para calcular los coeficientes a, b,c y d
     for i in 0..m-1
        ax[i] = x[i]
        bx[i] = der[i]
        cx[i] = 3*(x[i+1]-x[i])-2*der[i]-der[i+1]
        dx[i] = 2*(x[i]-x[i+1])+der[i]+der[i+1]
     end
     #----------------------------------------------------------y
     
     # Calculamos el valor de delta para y 
     delta[0] = 3*(y[1]-y[0])*omega[0]
     for i in 1..m-1
        delta[i] = (3*(y[i+1]-y[i-1])-delta[i-1])*omega[i]
     end
     delta[m] = (3*(y[m]-y[m-1])-delta[m-1])*omega[m]
     # Valor de la primera derivada en los puntos (eje y) 
     der[m] = delta[m]
     for i in 0..m-1
        der[m-1-i] = delta[m-1-i]-omega[m-1-i]*der[m-i]
     end
     
     ay=[], by=[], cy=[], dy=[]
     # Sustituimos los valores de omega, delta y la primera derivada
     # para calcular los coeficientes a, b,c y d
     for i in 0..m-1
        ay[i] = y[i]
        by[i] = der[i]
        cy[i] = 3*(y[i+1]-y[i])-2*der[i]-der[i+1]
        dy[i] = 2*(y[i]-y[i+1])+der[i]+der[i+1]
     end
     
     #----------------------------------------------------------z
     # Calculamos el valor de delta para z 
     delta[0] = 3*(z[1]-z[0])*omega[0]
     for i in 1..m-1
        delta[i] = (3*(z[i+1]-z[i-1])-delta[i-1])*omega[i]
     end
     delta[m] = (3*(z[m]-z[m-1])-delta[m-1])*omega[m]
     
     # Valor de la primera derivada en los puntos (eje z) 
     der[m] = delta[m]
     for i in 0..m-1
        der[m-1-i] = delta[m-1-i]-omega[m-1-i]*der[m-i]
     end
     
     az=[], bz=[], cz=[], dz=[]
     # Sustituimos los valores de omega, delta z la primera derivada
     # para calcular los coeficientes a, b,c z d
     for i in 0..m-1
        az[i] = z[i]
        bz[i] = der[i]
        cz[i] = 3*(z[i+1]-z[i])-2*der[i]-der[i+1]
        dz[i] = 2*(z[i]-z[i+1])+der[i]+der[i+1]
     end
     #------------------------------------------------------------
     
     # evaluar los polinomios obtenidos
     dt = 1.0/ num_seg;
     pts_s=[], sx=0, sy=0, sz=0, pts_spline=[]
     pts_s.clear
     for i in 0..m-1
        0.step(1,dt) do |t|
           sx = ax[i] + bx[i]*t + cx[i]*t*t + dx[i]*t*t*t
           sy = ay[i] + by[i]*t + cy[i]*t*t + dy[i]*t*t*t
           sz = az[i] + bz[i]*t + cz[i]*t*t + dz[i]*t*t*t
           pts_s.push(Geom::Point3d.new(sx,sy,sz))
        end
        sx = ax[i] + bx[i] + cx[i] + dx[i]
        sy = ay[i] + by[i] + cy[i] + dy[i]
        sz = az[i] + bz[i] + cz[i] + dz[i]
        pts_s.push(Geom::Point3d.new(sx,sy,sz))
     end
     
     # Obtener intervalos regulares
     spline = self.p_regulares2(pts_s)
     n = spline.size-1
     
     # Crear transformaciones
     tr=[],v1=[],v2=[],v3=[]
     if es_linea
        ve1 = Geom::Vector3d.new(spline[0].to_a)
        ve2 = Geom::Vector3d.new(spline.last.to_a)
        if (ve2-ve1).parallel? Z_AXIS
           tr[0] = Geom::Transformation.new spline[0],X_AXIS,Z_AXIS
        else
           vec_x = Geom::Vector3d.new(-((ve2-ve1).y),(ve2-ve1).x,0) 
           tr[0] = Geom::Transformation.new spline[0],vec_x,ve2-ve1
        end
        for i in 1..n
           tr[i] = Geom::Transformation.new spline[i],tr[0].xaxis,tr[0].yaxis
        end
     else
        for i in 1..n-1
           v1[i] = Geom::Vector3d.new(spline[i-1]-spline[i])
           v2[i] = Geom::Vector3d.new(spline[i+1]-spline[i])
        end
        for i in 1..n-1
           v3[i] = v1[i]*v2[i]
           tr[i] = false
           if (v3[i]).length.to_f > 0
              tr[i] = true 
              break
           end   
        end 
        vec_x = v3[tr.index(true)]
        vec_y = spline[1]-spline[0]
        vec_y = spline[1]-spline[n-1] if @closed && r1==r2 && vuel-vuel.to_i==0
        tr[0] = Geom::Transformation.new(spline[0],vec_x,vec_y)
        for i in 1..n-1
           zona = v1[i].dot v2[i]
           if zona < 0 #significa angulo entre vectores mayor a 90
              vec_x = (v2[i]*tr[i-1].xaxis)*v2[i]
              tr[i] = Geom::Transformation.new(spline[i],vec_x,v2[i])
           elsif zona > 0 #significa angulo entre vectores menor a 90
              vec_x = (v2[i]*tr[i-1].xaxis)*v2[i]
              tr1 = Geom::Transformation.rotation([0,0,0],v1[i]*v2[i],Math::PI)
              vec_x.transform! tr1
              tr[i] = Geom::Transformation.new(spline[i],vec_x,v2[i])
           else
              tr1 = Geom::Transformation.rotation([0,0,0],v2[i]*v1[i],Math::PI/ 2)
              vec_x = tr[i-1].xaxis
              vec_x = vec_x.transform! tr1
           end
        end
        vec_x = tr[n-1].xaxis
        vec_y = v2[n-1]-v1[n-1]
        vec_y = v2[1]-v1[n-1] if @closed && r1==r2 && vuel-vuel.to_i==0
        tr[n] = Geom::Transformation.new(spline.last, vec_x, vec_y)
     end
     
     # para cerrar la helice
     theta=0, helix=[]
     if @closed 
        u_eje_x = tr[tr.size-1].xaxis.clone
        u_eje_x1 = u_eje_x.clone
        desfase = tr[0].xaxis.angle_between u_eje_x
        tri = Geom::Transformation.rotation([0,0,0],tr[tr.size-1].yaxis,-desfase)
        tri1 = Geom::Transformation.rotation([0,0,0],tr[tr.size-1].yaxis,desfase)
        u_eje_x.transform! tri
        u_eje_x1.transform! tri1
        an1 = tr[0].xaxis.angle_between u_eje_x
        an2 = tr[0].xaxis.angle_between u_eje_x1
        if an1<an2
           vueltas = vueltas-(desfase/ (2*Math::PI))
        else
           vueltas = vueltas+(desfase/ (2*Math::PI))
        end
     end
     Sketchup::set_status_text @lw[19]+'  | * * - - - |', SB_PROMPT
     
     # generar radios
     r = []
     r.clear
     m = (r2-r1).to_f/ n
     for i in 0..n
        r[i] = m*i + r1
     end
     
     # generar y aplicar factores randon suaves
     if vueltas >= 2 && random_suave>0
        x1 = 0
        n1 = vueltas.to_i-1
        seg_v = (@new_segment/ vueltas).to_i 
        for i in 1..n1
           r[i*seg_v] = r[i*seg_v]+r[i*seg_v]*(2*random_suave*rand()-random_suave)
           a1 = (i-1)*seg_v+1
           b1 = i*seg_v-1
           c1 = r[i*seg_v]-r[(i-1)*seg_v]
           for j in a1..b1
              x1 = (j+1-a1)/ seg_v.to_f
              r[j] =  r[(i-1)*seg_v] + c1*x1*x1*(3-2*x1) 
           end
        end
        a1 = n1*seg_v+1
        b1 = (n1+1)*seg_v-1
        c1 = r[(n1+1)*seg_v]-r[n1*seg_v]
        for j in a1..b1
           x1 = (j+1-a1)/ seg_v.to_f
           r[j] =  r[n1*seg_v] + c1*x1*x1*(3-2*x1) 
        end
     end
     
     # generar helice
     for i in 0..n
        theta = (2*Math::PI*vueltas*sentido_g*i/ n) + fase
        helix[i] = Geom::Point3d.new(r[i]*Math.sin(theta),0,r[i]*Math.cos(theta))
        helix[i].transform! tr[i]
     end
     helix[helix.size-1] = helix[0].clone if @closed && vuel-vuel.to_i==0 && r1==r2
     # determina si hay conflicto con el radio del tubo y .followme
     cambiar_rt = false
     cambiar_lt = false
     imposible_crear_tubo = false
     crear_tubo = false if radio_tubo == 0
     if crear_tubo
        vn = []
        v1 = Geom::Vector3d.new
        v2 = Geom::Vector3d.new
        if @closed
           v1 = Geom::Vector3d.new(helix[n]-helix[n-1]).normalize!
           v2 = Geom::Vector3d.new(helix[1]-helix[0]).normalize!
           vn[0] = v1+v2
           vn[n] = v1+v2
        else
           vn[0] = Geom::Vector3d.new(helix[1]-helix[0]).normalize!
           vn[n] = Geom::Vector3d.new(helix[n]-helix[n-1]).normalize!
        end
        for i in 1..n-1
           v1 = Geom::Vector3d.new(helix[i]-helix[i-1]).normalize!
           v2 = Geom::Vector3d.new(helix[i+1]-helix[i]).normalize!
           vn[i] = v1+v2
        end
        l_21 = []
        l_21.clear
        r_critic = []
        r_critic.clear
        for i in 0..n-1
           plane1 = [helix[i],vn[i]]
           plane2 = [helix[i+1],vn[i+1]]
           recta2 = Geom.intersect_plane_plane(plane1, plane2)
           if recta2
              vec_21 = Geom::Vector3d.new(helix[i+1]-helix[i])
              recta1 = [helix[i],vec_21]
              vec_ab = Geom::Vector3d.new(recta2[0]-recta1[0])
              u_cruz_v = recta1[1]*recta2[1]
              det = vec_ab.x*u_cruz_v.x + vec_ab.y*u_cruz_v.y + vec_ab.z*u_cruz_v.z
              r_critic << det.abs/ (u_cruz_v.length.to_f)
              l_21 << vec_21
           end
        end
        if r_critic.size > 0
           indi = r_critic.index(r_critic.min)
           r_max = r_critic[indi] - r_critic[indi]*0.0003/ (l_21[indi].length)
        end
        if r_max
           if cambiar_rt = radio_tubo > r_max
              radio_tubo = r_max.to_l
              input[10] = (r_max-0.00001).to_l
           end
        end
        max_lados_tubo = (Math::PI/ (Math.asin(0.0005/ (2*radio_tubo)))-1).round
        if cambiar_lt = lados_tubo > max_lados_tubo
           if max_lados_tubo >= 3 
             lados_tubo = max_lados_tubo
             input[11] = lados_tubo
           else
              crear_tubo = false
              cambiar_rt = false
              cambiar_lt = false
              imposible_crear_tubo = true
           end
        end   
     end # if crear_tubo
     
     #-------------------------------------------------------------------------
     
     model.set_attribute "Valor Defecto #{@len} #{@version}", "Lista Defecto", input
     begin
        if Sketchup.version.to_f < 7.0
           model.start_operation("Draw-HelixD")
        else
           model.start_operation("Draw-HelixD", true)
        end
        grupo = ent.add_group
        entities = grupo.entities
        laCurva = entities.add_curve helix
        Sketchup::set_status_text @lw[19]+'  | * * * - - |', SB_PROMPT
        if crear_tubo
           normal_cir = (helix[0].vector_to helix[1]).normalize
           elCircle = entities.add_circle helix[0], normal_cir, radio_tubo, lados_tubo
           laCara = entities.add_face elCircle
           laCara.reverse!.followme laCurva
        end
        Sketchup::set_status_text @lw[20]+'  | * * * * * |', SB_PROMPT
        #entities.add_curve spline
        UI.messagebox "\n"+@lw[21]+" "+input[10].to_s+".\n\n"+@lw[22] if cambiar_rt
        UI.messagebox "\n"+@lw[23]+" "+input[11].to_s+".\n\n"+@lw[24] if cambiar_lt
        UI.messagebox "\n"+@lw[25]+@lw[26] if imposible_crear_tubo
        Sketchup::set_status_text @lw[20]+'  | * * * * * |', SB_VCB_VALUE
        model.commit_operation
     rescue
        model.abort_operation
     ensure
        #reset
     end
     
  end # def self.dibujar_helice
  
end # module HelixD
